﻿using C1.Chart;
using C1.Win.C1Input;
using C1.Win.Chart;
using C1.Win.FlexPivot;
using FlexPivot_Data;
using FlexPivot_Data.Model;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace FlexPivot_Demo
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            // fills country combobox with country names
            cbCountry.ItemsDataSource = new string[] { "World", "USA", "Brazil", "Russia", "Spain", "UK", "India", "Italy", "Peru", "Germany", "Iran", "Turkey", "France", "China"};

            cbCountry.SelectedItemChanged += CbCountry_SelectedItemChanged;
            cbCountry.SelectedIndex = 0;

            // prevents FlexPivot from showing totals row in FlexPivotGrid
            flexPivotPage.ShowTotalsRows =  C1.FlexPivot.ShowTotals.None;
            
            // sets FlexChart's color scheme
            flexPivotPage.Palette = Palette.Darkly;

            #region Define FlexPivotChart settings

            // sets FlexPivotChart type
            flexPivotPage.FlexPivotChart.ChartType = C1.Win.FlexPivot.ChartType.Column;

            // accesses new chart 'FlexChart' in FlexPivotPage
            var flexChart = flexPivotPage.FlexPivotChart.Chart as FlexChart;
            if (flexChart != null)
            {
                // gets FlexChart's Series collection
                var seriesColl = flexChart.Series as ObservableCollection<Series>;
                seriesColl.CollectionChanged += SeriesColl_CollectionChanged;
            }

            // allows toggling series visibility by clicking the legend
            flexChart.LegendToggle = true;

            // enables showing AxisScrollbar
            flexPivotPage.FlexPivotChart.UseAxisScrollbar = true;

            //enables data point selection in FlexChart(to get associated details for its data point)
            flexChart.SelectionMode = ChartSelectionMode.Point;

            flexChart.Rendering += (s, e) =>
            {
                // sets label's angle for AxisX
                flexChart.AxisX.LabelAngle = 45;
            };
            flexChart.Rendered += (s, e) =>
            {
                // sets Legend's position
                flexChart.Legend.Position = Position.Top;

                // draws selection for zooming
                if (!_start.IsEmpty && !_last.IsEmpty)
                {
                    var p1 = _start;
                    var p2 = _last;
                    e.Engine.SetFill(null);
                    e.Engine.SetStroke(Brushes.Red);
                    e.Engine.SetStrokeThickness(1);
                    e.Engine.SetStrokePattern(new double[] { 3, 3 });
                    e.Engine.DrawRect(Math.Min(p1.X, p2.X), Math.Min(p1.Y, p2.Y), Math.Abs(p2.X - p1.X), Math.Abs(p2.Y - p1.Y));
                }

                // redraws FlexChart
                flexChart.Invalidate();
            };

            // handles FlexChart's mouse move event to modify its ToolTip & zooming
            flexChart.MouseDown += FlexChart_MouseDown;
            flexChart.MouseMove += Chart_MouseMove;
            flexChart.MouseUp += FlexChart_MouseUp;
            #endregion
        }
        #region Zooming
        Point _start, _last;
        bool _IsMouseDown;
        private void FlexChart_MouseUp(object sender, MouseEventArgs e)
        {
            var flexChart = sender as FlexChart;

            // resets zooming in FlexChart
            if(e.Button== MouseButtons.Right)
            {
                flexChart.AxisX.Min = flexChart.AxisX.Max = double.NaN;
                flexChart.AxisY.Min = flexChart.AxisY.Max = double.NaN;
            }

            // stops Zooming
            _IsMouseDown = false;
            if (!_last.IsEmpty)
            {
                var start = flexChart.PointToData(_start);
                var last = flexChart.PointToData(_last);

                // updates axes with new limits
                flexChart.AxisX.Min = Math.Min(start.X, last.X);
                flexChart.AxisX.Max = Math.Max(start.X, last.X);
                flexChart.AxisY.Min = Math.Min(start.Y, last.Y);
                flexChart.AxisY.Max = Math.Max(start.Y, last.Y);
            }
            // clean up
            _start = _last = Point.Empty;
        }
        #endregion

        #region Selecting country for which to view Covid19 information in FlexPivot
        private void CbCountry_SelectedItemChanged(object sender, EventArgs e)
        {
            var countryCmb = sender as C1ComboBox;
            if (countryCmb != null && countryCmb.SelectedItem.ToString() != string.Empty)
            {
                ChangeDataSource(countryCmb.SelectedItem.ToString());
            }
        }
        #endregion

        #region Modify individual FlexChart Series
        private void SeriesColl_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            // handles only if a Series is added to FlexChart
            if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Add)
            {
                var series = e.NewItems[e.NewItems.Count - 1] as Series;
                if (series == null)
                    return;
                 switch (series.Name)
                {
                    case "TotalCases":
                        series.ChartType = C1.Chart.ChartType.Line;
                        series.AxisY = new Axis()
                        {
                            AxisLine = true,
                            Position = C1.Chart.Position.Left,
                            Title = "Total patients diagnosed +ve with Covid-19",
                            Format = "0,0"
                        };
                        break;
                    case "NewCases":
                        series.ChartType = C1.Chart.ChartType.Step;
                        series.AxisY = new Axis()
                        {
                            Position = C1.Chart.Position.Left,
                            AxisLine = true,
                            Title = "Daily new patients diagnosed +ve with Covid-19"
                        };
                        break;
                    case "TotalDeaths":
                        series.ChartType = C1.Chart.ChartType.Column;
                        series.AxisY = new Axis()
                        {
                            AxisLine = true,
                            Position = C1.Chart.Position.Right,
                            Title = "Total deaths due to Covid-19",
                            Format = "0,0"
                        };
                        break;
                    case "NewDeaths":
                        series.ChartType = C1.Chart.ChartType.Column;
                        series.AxisY = new Axis()
                        {
                            Position = C1.Chart.Position.Right,
                            AxisLine = true,
                            Title = "New deaths due to Covid-19"
                        };
                        break;
                }
            }
        }
        #endregion

        #region Modify FlexChart ToolTips & Zooming

        Point _selectedLocation = new Point(0, 0);

        // stores selected series point location (for showing tooltip later)
        private void FlexChart_MouseDown(object sender, MouseEventArgs e)
        {
            var flexChart = sender as FlexChart;

            #region Zooming specific code

            // starts Zooming
            _IsMouseDown = true;
            _start = e.Location;
            _last = Point.Empty;
            #endregion

            #region Tooltip specific code

            // gets element under mouse pointer
            var hitTestInfo = flexChart.HitTest(e.Location);
            if (hitTestInfo == null || hitTestInfo.Item == null)
                return;

            if (hitTestInfo.ChartElement == ChartElement.PlotArea)
            {
                _selectedLocation = e.Location;
            }
            #endregion
        }
        private void Chart_MouseMove(object sender, MouseEventArgs e)
        {
            var flexChart = sender as FlexChart;

            #region Zooming specific code

            // when zooming, update selection range
            if (_IsMouseDown)
            {
                var ptCurrent = e.Location;
                var left = (int)flexChart.PlotRect.Left;
                var right = (int)flexChart.PlotRect.Right;
                var top = (int)flexChart.PlotRect.Top;
                var bot = (int)flexChart.PlotRect.Bottom;
                ptCurrent.X = ptCurrent.X < left ? left : ptCurrent.X > right ? right : ptCurrent.X;
                ptCurrent.Y = ptCurrent.Y < top ? top : ptCurrent.Y > bot ? bot : ptCurrent.Y;
                _last = ptCurrent;
                flexChart.Refresh();
            }
            #endregion

            #region Tooltip specific code

            // gets element under mouse pointer
            var hitTestInfo = flexChart.HitTest(e.Location);
            if (hitTestInfo == null || hitTestInfo.Item == null)
                return;

            if (hitTestInfo.Item != null && hitTestInfo.Distance <= 3)
            {
                // gets date (AxisX label) specific to the mouse pointer
                var date = ((List<string>)flexChart.AxisX.DataSource)[hitTestInfo.PointIndex];

                // shows modified tooltip
                if (flexChart.SelectedIndex == -1 || !Enumerable.Range(_selectedLocation.X - 4, 9).Contains(e.Location.X))
                {

                    // modifies FlexChart tooltip value/contents
                    flexChart.ToolTip.Content = String.Format("Date: {0}\n{1}: {2}",
                          Convert.ToDateTime(date).ToString("d MMMM yyyy", CultureInfo.CreateSpecificCulture("en-US")),
                          hitTestInfo.Series.Name, String.Format("{0:n0}", hitTestInfo.Y));
                }
                // shows tooltip for the selected series point
                else
                {
                    // get details for selected element
                    CovidData selectedData = (((flexChart.Parent as FlexPivotChart).DataSource as C1FlexPivotPanel).DataSource as List<CovidData>).OrderBy(c => c.TotalCases).ToList()[flexChart.SelectedIndex] as CovidData;
                    if (selectedData != null)
                    {
                        var tooltipTemplate = @"**Date: {0}" + Environment.NewLine + "**Total Cases: {1}" + Environment.NewLine + "**Total Deaths: {2}" + Environment.NewLine + "**New Cases: {3}" + Environment.NewLine + "**New Deaths: {4}";

                        var tooltipContent = String.Format(tooltipTemplate, selectedData.Date.ToLongDateString(), selectedData.TotalCases, selectedData.TotalDeaths, selectedData.NewCases, selectedData.NewDeaths);
                        flexChart.ToolTip.Content = tooltipContent;
                    }
                }
            }
            #endregion
        }
        #endregion

        #region DataSource
        public void ChangeDataSource(string countryName)
        {
            var data = DataLoader.Import(countryName).OrderBy(p=>p.Date).ToList();
            flexPivotPage.DataSource = data;
            flexPivotPage.FlexPivotEngine.RowFields.Add("Date");
            flexPivotPage.FlexPivotEngine.ValueFields.Add("TotalCases");
            flexPivotPage.FlexPivotEngine.ValueFields.Add("NewCases");
            flexPivotPage.FlexPivotEngine.ValueFields.Add("TotalDeaths");
            flexPivotPage.FlexPivotEngine.ValueFields.Add("NewDeaths");

            if (countryName != "World")
                flexPivotPage.FlexPivotChart.ChartTitle = "Covid-19 Status for " + countryName;
            else
                flexPivotPage.FlexPivotChart.ChartTitle = "Covid-19 Status in World";
        }
        #endregion
    }
}
